var signer = require("ApiGateway/signer.js");
var pkg = require('../../../../package.json');
var iamAPI = require("./iam_api.json");
var url = require("url");


/**
 * 从pkg文件中读出iam的endpoint
 * @param pkg
 * @returns {*}
 */
function getIAMEndpoint(pkg) {
    try {
        return pkg['access']['endpoint']
    } catch (err) {
        logger.warn("get the iam info failed from package.json.");
        return "";
    }
}

const DOMAIN_CACHE_FILE = global.appDataPath + "domainInfo";

/**
 * 负责与iam连接，管理底层通信
 */
function IAMConnector()  {
    this.endpoint = getIAMEndpoint(pkg);  //package.json中可配置一个默认url
    this.ak = undefined;
    this.sk = undefined;
    this.agent = undefined;
    this.domainCache = {};    //缓存domain_id
    this.userCache = []; //[{domain_id:"123","name":"jianghcangle","id":"1"},{domain_id:"123","name":"1111","id":"2"}];    //useCache信息,是个数组，每一项是一个{domain_id,name,id}
    this.projectCache = []; //[{domain_id:"123","name":"cn-north-1","id":"1"},{domain_id:"123","name":"cn-north-2","id":"2"}];    //projectCache信息,是个数组，每一项是一个{domain_id,name,id}
    this.loadDomainCache();
};
IAMConnector.prototype.loadDomainCache = function(){
    var fs = require("fs");
    var self = this;
    fs.readFile(DOMAIN_CACHE_FILE, "utf8", function (err, data) {
            if (err) {
                logger.error("read DOMAIN_CACHE_FILE failed.", DOMAIN_CACHE_FILE, err);
                return;
            }
            try {
                var cache = JSON.parse(data);
                self.domainCache = cache;
            } catch (error) {
                logger.error("parse DOMAIN_CACHE_FILE failed. ", DOMAIN_CACHE_FILE, data, error);
                return;
            }
        }
    );
};

IAMConnector.prototype._saveDomainCache = function(){
    var fs = require("fs");
    var domainStr = JSON.stringify(this.domainCache);
    fs.writeFile(DOMAIN_CACHE_FILE, domainStr, function (err) {
        if (err) {
            logger.error("save domain info failed.", DOMAIN_CACHE_FILE, domainStr);
        }
    })
};


/**
 * 解析endpoint，从中得到host,port信息，并返回http调用库
 * @param endpoint
 * @param httpOption
 * @returns {Object}
 * @private
 */
IAMConnector.prototype._loadEndpoint = function(endpoint, httpOption){
    if (!endpoint) {
        endpoint = this.endpoint;
    }
    if (endpoint.startsWith("http")) {
        var urlObj = url.parse(endpoint);
        httpOption.host = urlObj.host;
    }
    var http = endpoint.startsWith("https") ? require("https") : require("http");
    return http;
};


/**
 * 如果切换账户，需要重新加载一下
 * @param accountInfo
 */
IAMConnector.prototype.loadUserInfo = function(accountInfo){
    if (accountInfo) {
        this.ak = accountInfo.accessKey;
        this.sk = accountInfo.secretKey;
    }
};

/**
 * 利用ak和sk进行签名
 * @param ak
 * @param sk
 * @param httpOption
 */
IAMConnector.prototype._signAKSK = function(httpOption){
    var sig = new signer.Signer();
    sig.AppKey = this.ak;
    sig.AppSecret = this.sk;
    var r = new signer.HttpRequest();
    r.host = httpOption.host;
    r.method = httpOption.method.toUpperCase();
    r.uri = httpOption.uri;

    if (httpOption.query) {
        r.query = httpOption.query;
    }
    if (httpOption.body) {
        r.body = httpOption.body;
    }
    r.headers = httpOption.headers;
    return sig.Sign(r);
};

IAMConnector.prototype.request = function(method, httpOption, callback){
    var operation = iamAPI.operations[method]
    var headers = operation.headers ? operation.headers : { 'Content-Type': 'application/json;charset=utf8', 'x-domain-name' : 'xxx'};
    httpOption.headers = headers;
    httpOption.method = operation.method;
    var host = operation.endpoint ? operation.endpoint : httpOption.host;
    httpOption.uri = operation.path ? operation.path : httpOption.path;
    var http = this._loadEndpoint(host, httpOption);

    var signObj = this._signAKSK(httpOption);
    signObj.agent = this.agent;
    signObj.rejectUnauthorized = false;
    var clientRequest = http.request(signObj, function (resp) {
        var chunks = [];
        resp.on('data', function (chunk) {
            chunks.push(chunk);
        });
        resp.on('end', function () {
            callback(null, Buffer.concat(chunks).toString("utf-8"));
        });
        resp.on('error', function (err) {
            callback(err, null);
        });
    });
    if (httpOption.body) {
        clientRequest.write(httpOption.body);
    }
    clientRequest.end();
};

IAMConnector.prototype.loadAgent = function(agent){
    if (agent) {
        this.agent = agent;
    } else {
        this.agent = undefined;
    }
};

/**
 * 查询ak sk所属租户下所有用户信息
 *
 * @param domainID ak sk所代表用户的domainid
 * @param callback   (err,data)   data:{users:[userInfo]} ,userInfo={domain_id,id,name}
 */
IAMConnector.prototype.queryUsers = function(domainID, callback){
    var self = this;
    if (typeof(domainID) === "function") {
        callback = domainID;
        domainID = undefined;
    }
    if (!domainID) {
        domainID = this.getDomainID();
    }
    self.request("getUserList", {"headers": {"X-Domain-Id": domainID}}, function (err, data) {
        if (data) {
            data = JSON.parse(data)
            if (data.error || data.message) {
                self.resetDomainID();
                return callback(data.error || new Error(data.message), null);
            }
            self.userCache = data.users ? data.users : [];
            self.setDomainID(domainID);
            this._saveDomainCache();
            return callback(err, data);
        }
        if (err) {
            self.resetDomainID();
        }
        return callback(err, data);
    });
};

/**
 * 查询所有的projects
 *
 * @param region 区域
 * @param callback   (err,data)   data:{projects:[projectInfo]} ,projectInfo={domain_id,id,name}
 */
IAMConnector.prototype.queryProjects = function(region, callback){
    var http = require('https');
    var sig = new signer.Signer();
    sig.AppKey = this.ak;
    sig.AppSecret = this.sk;

    var r = new signer.HttpRequest();
    r.host = 'iam.'+region+'.otc.t-systems.com'; //德电iam接口地址
    r.method = 'GET';
    r.uri = '/v3/projects';
    r.headers = {'Content-Type': 'application/json;charset=utf8', 'x-domain-name' : 'xxx'};

    var opt = sig.Sign(r);

    var req = http.request(opt, function (res) {

        global.logger.info(res.statusCode);
        global.logger.info('headers:',JSON.stringify(res.headers));
        var chunks = [];
        res.on('data', function (chunk) {
            chunks.push(chunk);
        });
        res.on('end', function () {
            callback(null, Buffer.concat(chunks).toString("utf-8"));
        });

    });
    req.on('error', function (err) {
        callback(err, null);
    });
    req.end();
};

/**
 * 查询指定用户可访问的projects
 *
 * @param userId 用户ID
 * @param domainID ak sk所代表用户的domainid
 * @param callback   (err,data)   data:{projects:[projectInfo]} ,projectInfo={domain_id,id,name}
 */
IAMConnector.prototype.queryUserProjects = function(userId, domainID, callback){
    var self = this;
    if (typeof(domainID) === "function") {
        callback = domainID;
        domainID = undefined;
    }
    if (!domainID) {
        domainID = this.getDomainID();
    }
    var path = "/v3/users/" + userId + "/projects";
    self.request("getUserProjectList", {"path": path,"headers": {"X-Domain-Id": domainID}}, function (err, data) {
        if (data) {
            data = JSON.parse(data)
            if (data.error || data.message) {
                self.resetDomainID();
                return callback(data.error || new Error(data.message), null);
            }
            self.userProjectCache = data.projects ? data.projects : [];
            self.setDomainID(domainID);
            self._saveDomainCache();
            return callback(err, data);
        }
        if (err) {
            self.resetDomainID();
        }
        return callback(err, data);
    });
};

/**
 * 查询project_id下所有的kms list
 *
 * @param region 区域
 * @param project_id 项目id
 * @param callback   (err,data)   data:{‘keys’：['1','2'], 'key_details':[kmsInfo]}
 */
IAMConnector.prototype.queryKmsList = function(region, project_id, callback){
    var http = require('https');
    var sig = new signer.Signer();
    sig.AppKey = this.ak;
    sig.AppSecret = this.sk;

    var r = new signer.HttpRequest();
    //r.host = 'iam.myhuaweicloud.com'; //华为云iam接口地址
    r.host = 'iam.'+region+'.otc.t-systems.com'; //德电iam接口地址
    r.method = 'POST';
    //r.uri = '/v3/projects';
    //r.uri = '/v1.0/cdn/domains';
    r.uri = '/v1.0/'+project_id+'/kms/list-keys';
    r.headers = {'Content-Type': 'application/json;charset=utf8', 'x-domain-name' : 'xxx'};

    var opt = sig.Sign(r);

    var req = http.request(opt, function (res) {

        global.logger.info(res.statusCode);
        global.logger.info('headers:',JSON.stringify(res.headers));
        var chunks = [];
        res.on('data', function (chunk) {
            chunks.push(chunk);
        });
        res.on('end', function () {
            callback(null, Buffer.concat(chunks).toString("utf-8"));
        });

    });
    req.on('error', function (err) {
        callback(err, null);
    });
    req.end();
};

IAMConnector.prototype.getDomainID = function getDomainID(){
    if (!this.domainCache[global.currentUserName]) {
        this.domainCache[global.currentUserName] = global.curOnwerId;
    }
    return this.domainCache[global.currentUserName] ? this.domainCache[global.currentUserName] : "";
};

IAMConnector.prototype.setDomainID = function(domainID){
    this.domainCache[global.currentUserName] = domainID;
};

//如果查询失败，则去除cache中的当前用户的domainID
IAMConnector.prototype.resetDomainID = function(){
    delete this.domainCache[global.currentUserName];
};


var accessConnector = new IAMConnector();
module.exports = accessConnector;
