var mongodb = require('mongodb');

var Const = require('./common/constants.js');
var collection;
//数据库重试次数
var retryCount = 3;
//索引标记
var COL_INDEX = 1;
//重试延迟时间，单位毫秒
var RETRY_DELAY = 2000;
//连接db

var server=undefined;
var _db=undefined;
function init_db(){
    global.logger.info("[MongoDB DAO] init mongodb and server");
    server = new mongodb.Server('localhost', 27117, {auto_reconnect: true, poolSize: 10});
    _db = new mongodb.Db('obs', server, {safe: true});
}


exports.open = function (callback) {
    if (global.db) {
        if (callback) {
            callback('', global.db);
        }
    } else {
        init_db();
        _db.open(function (err, db) {
            if (!err) {
                global.db = db;
            } else {
                global.logger.error("[MongoDB DAO] Open db error, ", err);
            }
            if (callback) {
                callback(err, db);
            }
        });
    }
};


//关闭db
exports.close = function (callback) {

    _db.close(function (err, db) {
        if (!err) {
            global.logger.info("[MongoDB DAO] Close db finish");
            global.db = false;
        } else {
            global.logger.error("[MongoDB DAO] Close db error, ", err);
        }
        if (callback) {
            callback(err, db);
        }
    });
};


exports.getCollection = function getCollection(callback) {
    var self = this;

    /**
     * 设置DB对应字段索引，获取Task表的数据
     * @param self 当前getCollection上下文
     * @param callback
     */
    function getTaskCollection(self, callback) {
        global.db.createCollection('tasks', {safe: true}, function (err, collection) {
            if (err) {
                global.logger.error("[MongoDB DAO] Create task collection error, ", err);
            } else {
                self.collection = collection;
                self.collection.ensureIndex({userId: COL_INDEX, stat: COL_INDEX}, {}, function (err) {});
                self.collection.ensureIndex({id: COL_INDEX}, {}, function (err) {});
                self.collection.ensureIndex({
                    bucket: COL_INDEX,
                    userId: COL_INDEX,
                    stat: COL_INDEX,
                    type: COL_INDEX
                }, {}, function (err) {});
                self.collection.ensureIndex({userId: COL_INDEX, stat: COL_INDEX, type: COL_INDEX}, {}, function (err) {});
                self.collection.ensureIndex({userId: COL_INDEX, type: COL_INDEX}, {}, function (err) {});
            }
            callback(err, collection);
        });
    }
    global.logger.info("[MongoDB DAO] Start create db connection")
    if (!global.db) {
        init_db();
        global.logger.info("[MongoDB DAO] Init DB Connection");
        try {
            _db.open(function (err, data) {
                if (err) {
                    if (global.db && global.db.openCalled) {
                        getTaskCollection(self, callback);
                        return;
                    }
                    setTimeout(function () {
                        //重试次数-1
                        if (retryCount-- < Const("NUM_ZERO")) {
                            global.logger.warn("[MongoDB DAO] Can't connect DB");
                            return;
                        }
                        global.logger.error("[MongoDB DAO] Open db error, retry...count=", retryCount, err);
                        self.getCollection(callback);
                    }, RETRY_DELAY);
                } else {
                    // 连接方式
                    global.db = data;
                    global.db.on("close",function(err){
                        global.logger.error("[MongoDB DAO] close event!", err)
                        global.db = undefined;
                        getCollection(function(){});
                    });
                    global.Tools.decode('mongoPwd', global.newMongoPwd, function (err, decipherText) {
                        global.db.authenticate('obs', decipherText, function (err, data) {
                            if(err) {
                                global.logger.error("auth failed.", err);
                                if (err.code == 18) {
                                    global.logger.warn("[MongoDB DAO] Auth DB User failed, add [obs] user", err);
                                    global.db.addUser('obs', decipherText, {
                                        dbName: 'obs',
                                        roles: ["readWrite"]
                                    }, function (err, data) {
                                        global.db.authenticate('obs', decipherText, function (err, data) {
                                            if(err)
                                            {
                                                global.logger.error("[MongoDB DAO]Add auth user failed.",err)
                                            }else{
                                                global.logger.info("[MongoDB DAO] Add auth user success");
                                                getTaskCollection(self, callback);
                                            }
                                        });
                                    });
                                }
                            } else {
                                global.logger.info("[MongoDB DAO] Auth user success");
                                getTaskCollection(self, callback);
                            }
                        });
                    });
                }
            });
        } catch (err) {
            if(err.message.indexOf("db object already connecting") > -1) {
                global.logger.warn("Connection is exist.", err);
                setTimeout(function(){
                    global.logger.warn("Connection is exist")
                    self.getCollection(callback);
                }, 500)

            } else {
                global.logger.error("Connection Error.", err);
            }
        }
    } else {
        global.logger.info("[MongoDB DAO] DB Connection has existed, it will directly to et collection.");
        getTaskCollection(self, callback);
    }


};


/**
 insert
 */
exports.add = function (task, callback) {
    var self = this;
    self.collection.insert([task], {safe: true}, function (err, result) {
        if (callback) {
            callback();
        }
    });
};

/**
 delete
 */
exports.del = function (task) {
    var self = this;
    self.collection.remove({_id: task._id}, {safe: true}, function (err, result) {
    });
};

/**
 clear
 */
exports.clear = function () {
    var self = this;
    self.collection.remove({}, {safe: true}, function (err, result) {

    });
};


/**
 delByQueryString
 */
exports.delByQueryString = function (queryString, callback) {
    var self = this;
    self.collection.remove(queryString, {safe: true}, function (err, result) {
        if (callback)
            callback();
    });
};

/**
 delete by id
 */
exports.delById = function (id, callback) {
    var self = this;
    self.collection.remove({id: id}, {safe: true}, function (err, result) {
        if (callback)
            callback(err, result);
    });
};

/**
 count
 */
exports.count = function (param, callback) {

    var self = this;
    if (!self.collection) {
        callback('collection is null', null);
        return;
    }


    if (param) param = param || {}
    self.collection.find(param).count(function (err, data) {
        if (callback)
            callback(err, data);
    });
};

exports.aggregate = function(param, callback) {
    var self = this;
    if (!self.collection) {
        callback('collection is null', null);
        return;
    }
    if (param) param = param || {}
    self.collection.aggregate(param, function (err, data) {
        if (callback)
            callback(err, data);
    });
}

/**
 list100
 */
exports.list100 = function (param, callback) {
    var self = this;
    if (param) param = param || {}
    self.collection.find(param).limit(100).toArray(function (err, data) {
        if (callback)
            callback(err, data);
    });
};


/**
 list
 param:querySring
 pageSize:分页条数
 pageNum：页数
 */
exports.list = function (param, pageSize, pageNum, callback) {
    var self = this;
    if (param) param = param || {}
    self.collection.find(param).skip((pageNum - 1) * pageSize).limit(pageSize).toArray(function (err, data) {
        if (callback)
            callback(err, data);
    });
};
/**
 * list排序的数据
 * @param param 查询条件
 * @param pageSize 每页大小
 * @param pageNum 页码
 * @param sortParam 排序字段
 * @param callback 回调
 */
exports.listWithSort = function (param, pageSize, pageNum, sortParam,callback) {
    var self = this;
    if (param) param = param || {}
    self.collection.find(param).sort(sortParam).skip((pageNum - 1) * pageSize).limit(pageSize).toArray(function (err, data) {
        if (err) {
            global.logger.err("Query db error, listWithSort", err);
        }
        if (callback)
            callback(err, data);
    });
};
/**
 lists
 param:querySring
 */
exports.lists = function (param, callback) {
    var self = this;
    if (param) param = param || {}
    self.collection.find(param).toArray(function (err, data) {
        if (callback)
            callback(err, data);
    });
};

/**
 get limit(1) task
 */
exports.get = function (query, callback) {
    var self = this;
    if (typeof query === 'function' && callback === undefined) {
        callback = query;
        query = null;
    }
    if (query) query = query || {}

    self.collection.findOne(query, function (err, data) {
        if (callback)
            callback(err, data);
    });

};


/**
 get limit(1) task and delete
 */
exports.getAndRemove = function (query, callback) {
    var self = this;
    if (typeof query === 'function' && callback === undefined) {
        callback = query;
        query = null;
    }
    if (query) query = query || {}

    self.collection.findAndRemove(query, function (err, data) {
        if (callback)
            callback(err, data);
    });
};

/**
 get limit(1) task and update ==> stat : "RUNNING"
 */
exports.getAndModify = function (query, callback) {
    var self = this;
    if (typeof query === 'function' && callback === undefined) {
        callback = query;
        query = null;
    }
    if (query) query = query || {}

    self.collection.findAndModify(query, null, {$set: {stat: "RUNNING"}}, {upsert: false}, function (err, data) {
        if (callback)
            callback(err, data);
    });
};


/**
 updata task stat
 */
exports.updateStat = function (task, callback) {
    var self = this;
    if (self.collection == undefined) {
        if (callback)
            callback(null, null);
        return;
    }
    self.collection.update({_id: task._id}, task, function (err, data) {
        if (callback)
            callback(err, data);
    });

};


exports.updateProperties = function (queryString, param, callback) {
    var self = this;
    if (self.collection == undefined) {
        if (callback)
            callback(null, null);
        return;
    }
    self.collection.update(queryString, param, {multi: true}, function (err, data) {
        if (callback)
            callback(err, data);
    });

};


/**
 distinct
 */
exports.distinct = function (keyString, queryString, callback) {
    var self = this;
    if (!self.collection) {
        callback('collection is null', null);
        return;
    }

    self.collection.distinct(keyString, queryString, function (err, data) {
        if (callback)
            callback(err, data);
    });
};

//模糊查询
exports.regexQuery = function (param, callback) {
    var self = this;
    if (param) param = param || {};
    self.collection.findOne(param, function (err, data) {
        if (callback)
            callback(err, data);
    });
};

/**
 delete
 */
exports.delSomeTasks = function (ids,call) {
    var self = this;
    self.collection.remove({id: {"$in":ids}}, {safe: true}, function (err, result) {
        call(err,result);
    });
};

/**
 listAndgetResult
 param:querySring
 result:需要返回的结果集
 */
exports.listAndGetResult = function (param,result, callback) {
    var self = this;
    if (param) param = param || {}
    self.collection.find(param,result).toArray(function (err, data) {
        if (callback)
            callback(err, data);
    });
};