// Load the SDK
var AWS = require('./lib/aws');
var s3; // s3 service, to be initialized in loaderUserInfo()



AWS.config['region'] = '';
AWS.config['signatureVersion'] = 's3';
AWS.config['s3ForcePathStyle'] = 'true';

/**
 * 初始化用户信息
 * @param userInfo
 */
AWS.loadUserInfo = function (userInfo){
    //更新用户信息和服务器信息
    AWS.config.accessKeyId = userInfo.accessKey;
    AWS.config.secretAccessKey = userInfo.secretKey;
    //加载enpoint信息
    if (userInfo.endpoint)
    {
        AWS.config.endpoint = userInfo.endpoint;
    }

    // Create an S3 client
    var service = new AWS.S3();

    service.config.update({
        //请求失败后最多重试次数
        maxRetries : 1,
        httpOptions: {
            //HTTP请求超时时间，30s
            timeout : 30000,
            //默认支持HTTPS，接受自签名证书
            rejectUnauthorized: false,
            agent: false,
            //如有反向代理，添加代理转发规则前缀
            proxy_prefix : userInfo.proxy_prefix
        }
    }, true);

    return service;
};

//==========桶操作接口===============

/*
 * 列举桶
 * 必须带的参数：callback
 */
AWS.listBuckets = function(callback)
{
    s3.listBuckets(function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 创建桶
 * 必须带的参数：callback, bucketName
 * region: 区域，由框架查询得到的预定义字符串
 */
AWS.createBucket = function(callback, bucketName, region, acl)
{
    var params = {
        Bucket : bucketName
    };

    if (acl)
    {
        params.ACL = acl;
    }

     if (region)
    {
        params.CreateBucketConfiguration = {LocationConstraint : region};
    }


    s3.createBucket(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 删除桶
 * 必须带的参数：callback, bucketName
 */
AWS.deleteBucket = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };
    s3.deleteBucket(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 获取桶区域信息
 * 必须带的参数：callback, bucketName
 * params: 可选的传入参数集合，在并发多个请求时才用到
 * handleIndex: 当前处理的数组内索引，用于传入回调函数
 */
AWS.getBucketLocation = function(callback, bucketName, myParams, handleIndex)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketLocation(params, function(err, data)
    {
        if (callback)
        {
            data = data || {};
            //把bucketName,handleIndex传给回调函数，便于异步跟踪处理
            data.bucketName = bucketName;
            data.handleIndex = handleIndex;
            data.err = err;

            callback(myParams, data);
        }

    });
};

//最多同时发送请求数
var MAX_SEND_THREADS = 5;

/*
 * 列举桶并获取桶的区域信息
 * 必须带的参数：callback
 */
AWS.listBucketsWithLocation = function(callback)
{
    s3.listBuckets(function(err, data)
    {
        var buckets = new Array();
        if (err)
        {
            if (callback)
            {
                callback(err, buckets);
            }
        }
        else
        {
            var names = "Got buckets [";
            for ( var index in data.Buckets)
            {
                var bucket = data.Buckets[index];
                buckets.push(bucket);

                names += (bucket.Name + ", ");
            }
            names += "]";

            //未有发送请求时，todos初始化为-1而不是0；
            var params = {origCallback : callback,
                buckets : buckets,
                todos : -1,
                currentIndex : buckets.length - 1
            };

            //最多同时发送5个异步请求
            threadPool(getNextBucketLocation, MAX_SEND_THREADS, params)

        }

    });
};

/**
 * 获取下一个桶区域信息
 * @param params
 * @param data1:
 */
function getNextBucketLocation(params, data1)
{
    var origCallback = params.origCallback;
    var buckets = params.buckets;

    //如果是得到response后的回调，则处理数据
    if (data1)
    {
        params.todos--;
        var bucketName = data1.bucketName;
        var index = data1.handleIndex;
        var location = data1.err ? '' : data1.LocationConstraint;

        buckets[index].LocationConstraint = location;

    }

    //获取到最后一个response，返回最终结果给原始回调
    if (params.todos === 0 && origCallback) {
        origCallback(data1.err, buckets);
    }

    //发送下一条request
    if (params.currentIndex >=0)
    {
        if (params.todos < 0)
        {
            //发送的第一条请求
            params.todos = 1;
        }
        else
        {
            //每发送一条请求+1，得到回调时-1
            params.todos++;
        }

        var handleIndex = params.currentIndex;
        params.currentIndex--;
        AWS.getBucketLocation(getNextBucketLocation, buckets[handleIndex].Name, params, handleIndex);
    }
}

/**
 * 一个简单的模拟线程池
 * @param doNext: 自回调函数，得到结果后再继续调用自身发送下一个请求
 * @param maxThreads: 并发请求数
 * @param params: 传入参数集合
 */
function threadPool(doNext, maxThreads, params)
{
    for (var i=0; i<maxThreads; i++)
    {
        doNext(params);
    }
}

//==========域名绑定操作接口===============

/*
 * 添加桶 Domain，每个桶最多支持5个domain
 * 必须带的参数：callback, bucketName, domainName
 */
AWS.putBucketDomain = function(callback, bucketName, domainName)
{
    var params = {
        Bucket : bucketName,
        DomainName : domainName
    };

    s3.putBucketDomain(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 获取桶 Domain 列表
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketDomains = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };
    s3.getBucketDomains(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 删除桶其中一个 Domain
 * 必须带的参数：callback, bucketName, domainName
 */
AWS.deleteBucketDomain = function(callback, bucketName, domainName)
{
    var params = {
        Bucket : bucketName,
        DomainName : domainName
    };
    s3.deleteBucketDomain(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

//==========Website操作接口===============

/*
 * 设置桶 Website
 * 必须带的参数：callback, bucketName, websiteConfiguration
 */
AWS.putBucketWebsite = function(callback, bucketName, websiteConfiguration)
{
    var params = {
        Bucket : bucketName,
        WebsiteConfiguration : websiteConfiguration
    };

    s3.putBucketWebsite(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 获取桶 Website
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketWebsite = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketWebsite(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};


/*
 * 删除桶 Website
 * 必须带的参数：callback, bucketName
 */
AWS.deleteBucketWebsite = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };
    s3.deleteBucketWebsite(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};


//==========对象操作接口===============

/*
 * 列举桶内对象
 * 必须带的参数：callback, bucketName
 * prefix: 如"myfolder1/",则只列举该目录下的对象以及直接子目录
 * maxKeys: 返回条目数最大值，超过1000则只取1000；不设置则默认值为1000
 * marker: 对象名排序后从该字符串开始往后取
 */
AWS.listObjects  = function(callback, bucketName, prefix, maxKeys, marker)
{
    var params = {
        Bucket : bucketName,
        Delimiter : '/'     //带上此参数，则只取回当前目录下的对象和子目录，不会得到子目录下包含的内容
    };

    if (prefix)
    {
        params.Prefix = prefix;
    }

    if (maxKeys)
    {
        params.MaxKeys = maxKeys;
    }

    if (marker)
    {
        params.Marker = marker;
    }

    s3.listObjects(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 创建目录
 * 必须带的参数：callback, bucketName, folderName
 * folderName: 为"folder1/"或者是"folder1/folder2/folder3/"这样的形式
 */
AWS.createFolder = function(callback, bucketName, folderName, acl)
{
    if (!folderName)
    {
        return;
    }

    var index = folderName.lastIndexOf('/');
    if (index !== folderName.length - 1)
    {
        folderName += '/';
    }

    var params = {
        Bucket : bucketName,
        Key : folderName
    };

    if (acl)
    {
        params.ACL = acl;
    }

    s3.putObject(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 上传对象
 * 必须带的参数：callback, bucketName, objName
 * objName: 为"abc.txt"或者是"folder1/folder2/abc.txt"这样的形式
 */
AWS.putObject = function(callback, bucketName, objName, fileContent, acl)
{

    var params = {
        Bucket : bucketName,
        Key : objName,
        Body : fileContent
    };

    if (acl)
    {
        params.ACL = acl;
    }

    s3.putObject(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });
};

/*
 * 获取下载对象URL
 * 必须带的参数：callback, bucketName, objName
 * objName: 为"abc.txt"或者是"folder1/folder2/abc.txt"这样的形式
 * return: 可供browser下载对象用的url
 */
AWS.getObjectUrl = function(callback, bucketName, objName)
{
    var params = {
        Bucket : bucketName,
        Key : objName
    };

    s3.getSignedUrl('getObject', params, function(err, data)
    {
        if (data)
        {
            var proxy = AWS.config['proxy_prefix'] || '';
            data = data.replace(AWS.config['endpoint'], AWS.config['endpoint'] + proxy);
        }

        if (callback)
        {
            callback(err, data);
        }
    });
};

/*
 * 下载对象
 * 必须带的参数：callback, bucketName, objName
 * objName: 为"abc.txt"或者是"folder1/folder2/abc.txt"这样的形式
 * return: 对象内容
 */
AWS.getObject = function(callback, bucketName, objName)
{
    var params = {
        Bucket : bucketName,
        Key : objName
    };

    s3.getObject(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 删除对象
 * 必须带的参数：callback, bucketName, objName
 * objName: 为"abc.txt"或者是"folder1/folder2/abc.txt"这样的形式
 */
AWS.deleteObject = function(callback, bucketName, objName)
{

    var params = {
        Bucket : bucketName,
        Key : objName
    };

    s3.deleteObject(params, function(err, data) {
        if (callback)
        {
            callback(err, data, objName);
        }

    });

};

/*
 * 获取桶ACL(access control list)
 * 必须带的参数：callback, bucketName,
 */
AWS.getBucketAcl = function(callback, bucketName)
{

    var params = {
        Bucket : bucketName
    };

    s3.getBucketAcl(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 更改桶ACL(access control list)
 * 必须带的参数：callback, bucketName
 */
AWS.putBucketAcl = function(callback, bucketName, acl, accessControlPolicy)
{

    var params = {
        Bucket : bucketName
    };

    //header里的x-amz-acl字符串
    if (acl)
    {
        params.ACL = acl;
    }

    //xml形式的body；与header二者取其一？
    if (accessControlPolicy)
    {
        params.AccessControlPolicy = accessControlPolicy;
    }

    s3.putBucketAcl(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取对象ACL(access control list)
 * 必须带的参数：callback, bucketName, objName
 * objName: 为"folder1/"或者是"folder1/folder2/abc.txt"这样的形式
 */
AWS.getObjectAcl = function(callback, bucketName, objName)
{

    var params = {
        Bucket : bucketName,
        Key : objName
    };

    s3.getObjectAcl(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 更改对象ACL(access control list)
 * 必须带的参数：callback, bucketName, objName
 * objName: 为"abc.txt"或者是"folder1/folder2/abc.txt"这样的形式
 */
AWS.putObjectAcl = function(callback, bucketName, objName, acl, accessControlPolicy)
{

    var params = {
        Bucket : bucketName,
        Key : objName
    };

    //header里的x-amz-acl字符串
    if (acl)
    {
        params.ACL = acl;
    }

    //xml形式的body；与header二者取其一？
    if (accessControlPolicy)
    {
        params.AccessControlPolicy = accessControlPolicy;
    }

    s3.putObjectAcl(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

//================跨域控制CORS=============
/*
 * 上传桶的CORS配置
 * 必须带的参数：callback, bucketName
 * corsConfiguration: 内含CORSRules，如与已有id相同则覆盖旧规则
 */
AWS.putBucketCors = function(callback, bucketName, corsConfiguration)
{
    var params = {
        Bucket : bucketName
    };

    //xml形式的body
    if (corsConfiguration)
    {
        params.CORSConfiguration = corsConfiguration;
    }

    s3.putBucketCors(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取桶的CORS配置
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketCors = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketCors(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 *
 * 修改存儲類別
 */
AWS.putBucketStoragePolicy = function(callback, bucketName, StoragePolicy)
{
    var params = {
        Bucket: bucketName,
        StoragePolicy: {
            DefaultStorageClass: StoragePolicy
        }
    }

    s3.putBucketStoragePolicy(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 删除桶的CORS配置
 * 必须带的参数：callback, bucketName
 */
AWS.deleteBucketCors = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.deleteBucketCors(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

//================配置桶的 Lifecycle=============
/*
 * 上传桶的 Lifecycle 配置
 * 必须带的参数：callback, bucketName
 * lifecycleConfiguration: 内含一条或多条Rule，每条Rule必须带Prefix和Status
 * 注意: Rules里的过期时间Days和Date方式二选一，例如
 * { Rules: [{Prefix: "folder1", Status: "Enabled", Expiration: {Date: "2015-05-30T00:00:00.000Z"} }]}
 */
AWS.putBucketLifecycle = function(callback, bucketName, lifecycleConfiguration)
{
    var params = {
        Bucket : bucketName
    };

    //xml形式的body
    if (lifecycleConfiguration)
    {
        params.LifecycleConfiguration = lifecycleConfiguration;
    }

    s3.putBucketLifecycle(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取桶的 Lifecycle 配置
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketLifecycle = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketLifecycle(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 删除桶的Lifecycle配置
 * 必须带的参数：callback, bucketName
 */
AWS.deleteBucketLifecycle = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.deleteBucketLifecycle(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

//================配置桶的 Logging =============
/*
 * 上传桶的 Logging 配置
 * 必须带的参数：callback, bucketName, bucketLoggingStatus
 * lifecycleConfiguration: 内含一条或多条Rule，每条Rule必须带Prefix和Status
 */
AWS.putBucketLogging = function(callback, bucketName, bucketLoggingStatus)
{
    var params = {
        Bucket : bucketName,
        BucketLoggingStatus : bucketLoggingStatus
    }

    s3.putBucketLogging(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取桶的 Logging 配置
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketLogging = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketLogging(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 删除桶的 Logging 配置
 * 必须带的参数：callback, bucketName
 */
AWS.deleteBucketLogging = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.deleteBucketLogging(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

//================配置桶的 Policy =============
/*
 * 上传桶的 Policy 配置
 * 必须带的参数：callback, bucketName, policy
 * policy: JSONString格式的桶策略描述，详见S3 API文档
 */
AWS.putBucketPolicy = function(callback, bucketName, policy)
{
    var params = {
        Bucket : bucketName,
        Policy : policy
    }

    s3.putBucketPolicy(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取桶的 Policy 配置
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketPolicy = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketPolicy(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 删除桶的 Policy 配置
 * 必须带的参数：callback, bucketName
 */
AWS.deleteBucketPolicy = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.deleteBucketPolicy(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取桶的 storageInfo
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketStorageInfo = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketStorageInfo(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

/*
 * 获取桶的 quota
 * 必须带的参数：callback, bucketName
 */
AWS.getBucketQuota = function(callback, bucketName)
{
    var params = {
        Bucket : bucketName
    };

    s3.getBucketQuota(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};


/*
 * 批量删除接口
 * 必须带的参数：callback, bucketName,objects
 */
AWS.deleteObjects = function(callback, bucketName,objects,quiet)
{
    var params = {
        Bucket : bucketName,
        Delete : { Objects : objects },
        Quiet : false
    };
    if(quiet){
        params.Quiet = true;
    }
    s3.deleteObjects(params, function(err, data)
    {
        if (callback)
        {
            callback(err, data);
        }

    });

};

